#!/usr/bin/python
import cups

def do_indent (indent):
	return "  "*indent

def getippqueue (dev, queue, depth):
	name = dev.rfind ('/')
	name = dev[name + 1:]
	dev = dev[6:]
	e = dev.find (':')
	if e == -1:
		e = dev.find ('/')
	host = dev[:e]
	cups.setServer (host)
	try:
		c = cups.Connection ()
		printers = c.getPrinters ()
		classes = c.getClasses ()
	except RuntimeError:
		# Failed to connect.
		return
	except cups.IPPError as e:
		if e == cups.IPP_OPERATION_NOT_SUPPORTED:
			# CUPS-Get-Printers not supported so not a CUPS server.
			printers = {}
			classes = {}
		else:
			return

	queue = c.getPrinterAttributes (name)
	dev = queue['device-uri']
	getqueue (name, queue, host, depth + 1, printers, classes)

def getqueue (name, queue, host, depth, printers, classes):
	indent = do_indent(depth)
	if queue['printer-type'] & cups.CUPS_PRINTER_CLASS:
		print ("%s* Name:\t%s[@%s] (class)" % (indent, name, host))
		dev = queue['device-uri']
		if dev.startswith ('ipp:'):
			getippqueue (dev, queue, depth)
		else:
			members = classes[name]
			depth += 1
			indent = do_indent(depth)
			for member in members:
				getqueue (member, printers[member], host,
					  depth, printers, classes)
	else:
		print ("%s* Name:\t%s[@%s]" % (indent, name, host))
		dev = queue['device-uri']
		info = queue['printer-info']
		print ("%sURI:\t%s" % (indent, dev))
		print ("%sInfo:\t%s" % (indent, info))
		if dev.startswith ('ipp:'):
			getippqueue (dev, name, depth)

	if depth == 0:
		print

def gethost (host=None, depth=0):
	if host:
		cups.setServer (host)
	else:
		host = "localhost"
	c = cups.Connection ()
	printers = c.getPrinters ()
	classes = c.getClasses ()
	indent = do_indent(depth)
	for name, queue in printers.items ():
		getqueue (name, queue, host, depth, printers, classes)

gethost()



#!/usr/bin/python

## Copyright (C) 2002, 2005, 2006, 2007, 2008, 2009, 2010, 2011, 2012, 2013, 2014  Red Hat, Inc
##  Author: Tim Waugh <twaugh@redhat.com>

## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.

## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.

## You should have received a copy of the GNU General Public License
## along with this program; if not, write to the Free Software
## Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

"""This is a set of Python bindings for the libcups library from the
CUPS project.

>>> # Example of getting a list of printers
>>> import cups
>>> conn = cups.Connection ()
>>> printers = conn.getPrinters ()
>>> for printer in printers:
...     print printer, printers[printer]["device-uri"]
...
HP ipp://192.168.1.1:631/printers/HP
duplex ipp://192.168.1.1:631/printers/duplex
HP-LaserJet-6MP ipp://192.168.1.1:631/printers/HP-LaserJet-6MP
EPSON-Stylus-D78 usb://EPSON/Stylus%20D78
"""

from distutils.core import setup, Extension
import sys
VERSION="1.9.73"
libraries=["cups"]

if sys.platform == "darwin" or sys.platform.startswith("freebsd"):
	libraries.append ("iconv")

setup (name="pycups",
       version=VERSION,
       description="Python bindings for libcups",
       long_description=__doc__,
       maintainer="Tim Waugh",
       maintainer_email="twaugh@redhat.com",
       url="http://cyberelk.net/tim/software/pycups/",
       download_url="http://cyberelk.net/tim/data/pycups/",
       classifiers=[
		"Intended Audience :: Developers",
		"Topic :: Software Development :: Libraries :: Python Modules",
		"License :: OSI Approved :: GNU General Public License (GPL)",
		"Development Status :: 5 - Production/Stable",
		"Operating System :: Unix",
		"Programming Language :: C",
		"Programming Language :: Python",
		"Programming Language :: Python :: 2",
		"Programming Language :: Python :: 3",
		],
       license="GPLv2+",
       ext_modules=[Extension("cups",
                              ["cupsmodule.c", "cupsconnection.c",
                               "cupsppd.c", "cupsipp.c"],
                              libraries=libraries,
                              define_macros=[("VERSION", '"%s"' % VERSION)])])



#!/usr/bin/python
import cups

# Simple demonstration of cups module

def callback (prompt):
	print ("Password is required for this operation")
	password = raw_input (prompt)
	return password

def test_cups_module ():
	cups.setUser ("root")
	cups.setPasswordCB (callback)
	conn = cups.Connection ()
	printers = list(conn.getPrinters ().keys ())

	if 0:
		print ("Getting cupsd.conf")
		file ("cupsd.conf", "w")
		conn.getFile ("/admin/conf/cupsd.conf", "cupsd.conf")
		print ("Putting cupsd.conf")
		conn.putFile ("/admin/conf/cupsd.conf", "cupsd.conf")

	print ("Getting PPD for %s" % printers[len (printers) - 1])
	f = conn.getPPD (printers[len (printers) - 1])
	ppd = cups.PPD (f)
	ppd.markDefaults ()
	print (ppd.conflicts ())
	groups = ppd.optionGroups
	for group in groups:
		for opt in group.options:
			print (list (map (lambda x: x["text"], opt.choices)))

test_cups_module ()
